/****
Fixed point computation, 8 bits are seen as the fractionnal part,
e.g. 0x100 = 1.0 .
(c)2009 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef FIXED_POINT_8__H
#define FIXED_POINT_8__H

/**
 * Number of bits necessary for converting between
 * regular int and fixed point values.
 **/
#define FIXED_POINT_8__FULL_BITS (256)

/**
 * Number of bits necessary for intermediary convertion
 * in multiplication.
 **/
#define FIXED_POINT_8__HALF_BITS (16)

/**
 * Fixed point value.
 **/
typedef int fp8 ;

/**
 * Intermediary fixed point value.
 **/
typedef int fph ;

/**
 * Convert a regular int value into a fixed point value.
 **/
fp8 FixedPoint8__new(int value)
{
	return (value * FIXED_POINT_8__FULL_BITS) ;
}

/**
 * Convert into an intermediate fixed point value for multiplication from a
 * fixed point value.
 **/
fph FixedPoint8__toHalf(fp8 value)
{
	return (value / FIXED_POINT_8__HALF_BITS) ;
}

/**
 * Convert into a regular int value from a fixed point value.
 **/
int FixedPoint8__toInt(fp8 value)
{
	return (value / FIXED_POINT_8__FULL_BITS) ;
}

/**
 * Addition.
 **/
fp8 FixedPoint8__add(fp8 x, fp8 y)
{
	return (x + y) ;
}

/**
 * Substraction.
 **/
fp8 FixedPoint8__sub(fp8 x, fp8 y)
{
	return (x - y) ;
}

/**
 * Multiplication.
 **/
fp8 FixedPoint8__mul(fp8 x, fp8 y)
{
	int ix = FixedPoint8__toInt(x);
	int fx = x - FixedPoint8__new(ix) ;
	int iy = FixedPoint8__toInt(y);
	int fy = y - FixedPoint8__new(iy) ;
	int ix_mul_iy = ix * iy ;
	int ix_mul_fy = ix * fy ;
	int iy_mul_fx = iy * fx ;
	int fx_mul_fy = fx * fy ;
	fp8 result = FixedPoint8__new(ix_mul_iy) ;
	result += FixedPoint8__add((fp8)ix_mul_fy, (fp8)iy_mul_fx) ;
	result += ((fp8) FixedPoint8__toInt((fp8)fx_mul_fy)) ;
	return result ;
}

/**
 * Division.
 **/
fp8 FixedPoint8__div(fp8 x, fp8 y)
{
	int _int_y = FixedPoint8__toInt(y) ;
	if(0 == _int_y)
	{
		/* we must avoid a divide by zero
		 * BUT it will lead to meaningless result if
		 * x is too high !!!
		 * */
		return (FixedPoint8__new(x) / y) ;
	}
	return (x / _int_y) ;
}

/**
 * Interpolation.
 * result = (1 - k)v1 + (k)v2
 * with k = position / range
 *
 * @param value1 value for position = 0
 * @param value2 value for position = range
 * @param range max value for position
 * @param position value between 0 and range
 */
fp8 FixedPoint8__interpolate(int value1, int value2, int range, int position)
{
	fp8 _k = FixedPoint8__new(position) ;
	fp8 _krange = FixedPoint8__new(range) ;
	fp8 _one_sub_k = FixedPoint8__sub(_krange, _k);
	fp8 _one_sub_k__mul__v1 = FixedPoint8__mul(_one_sub_k, FixedPoint8__new(value1));
	fp8 _k__mul__v2 = FixedPoint8__mul(_k, FixedPoint8__new(value2));
	/*fp8 _k = FixedPoint8__div(FixedPoint8__new(position), FixedPoint8__new(range)) ;
	fp8 _one_sub_k = FixedPoint8__sub(FixedPoint8__new(1), _k);
	fp8 _one_sub_k__mul__v1 = FixedPoint8__mul(_one_sub_k, FixedPoint8__new(value1));
	fp8 _k__mul__v2 = FixedPoint8__mul(_k, FixedPoint8__new(value2));
	*/
	/*int _partial_increment = (value2 - value1)*position ;
	fp8 _increment = FixedPoint8__div(FixedPoint8__new(_partial_increment), FixedPoint8__new(range)) ;
	*/
	fp8 _result = FixedPoint8__add(_one_sub_k__mul__v1, _k__mul__v2) ;
	_result /= range ;
	return _result; /*FixedPoint8__add(_one_sub_k__mul__v1, _k__mul__v2) ;*/
}

#endif /* ifndef FIXED_POINT_8__H */
