/****
HidTouch input driver.
(c)2008 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef HIDTOUCH__H
#define HIDTOUCH__H

/**
 * Symbols for xf86PostEventButton.
 **/
#define HDT__BUTTON_EVENT__LEFT_BUTTON (1)


/* ==== HdtRawData ==== */

/**
 * Symbols for raw data model.
 *
 * @see #HdtRawDataRec.pressure
 **/
#define HDT__RAW_DATA__IS_NOT_PRESSED (0)
#define HDT__RAW_DATA__IS_PRESSED     (1)

/**
 * Raw data model.
 *
 * Data read from the device and timestamped.
 **/
typedef struct _HdtRawData {
    /**
     * Timestamp of the data.
     *
     * will be used to compute tap sequence
     **/
    CARD32 timestamp ;

    /**
     * Pressure flag.
     *
     * HDT__RAW_DATA__IS_NOT_PRESSED or HDT__RAW_DATA__IS_PRESSED
     **/
    int pressure ;

    /**
     * X value (absolute).
     **/
    int x ;

    /**
     * Y value (absolute).
     **/
    int y ;
} HdtRawDataRec, *HdtRawDataPtr;

/* ==== API==== */

/**
 * Fill a raw data structure with data from the device.
 *
 * @param pInfo the device is retrieved from the InputInfo structure.
 * @param hdtRawData the structure to fill.
 **/
static void
HdtRawData__fillFromInputInfo(HdtRawDataPtr hdtRawData, InputInfoPtr pInfo) ;



/* ==== HdtOpCode ==== */

/**
 * OpCode (Operation Code) setup.
 *
 * Data read from the device consists in a few pairs of (opcode, value).
 * Significant opcode are specified here
 **/
typedef struct _HdtOpCode {
    /**
     * Pressure flag.
     **/
    int pressure ;

    /**
     * X value (absolute).
     **/
    int x ;

    /**
     * Y value (absolute).
     **/
    int y ;
} HdtOpCodeRec, *HdtOpCodePtr;

/* ==== API==== */

/* ==== HdtCalibration ==== */

/**
 * Calibration of the touch screen.
 **/
typedef struct _HdtCalibration {
    /**
     * Calibration Model.
     **/
    int model ;


    /**
     * Minimal X value.
     **/
    int minX ;

    /**
     * Minimal Y value.
     **/
    int minY ;

    /**
     * Maximal X value.
     **/
    int maxX ;

    /**
     * Maximal Y value.
     **/
    int maxY ;

    /**
     * Width of the screen.
     **/
    int screenWidth ;

    /**
     * Height of the screen.
     **/
    int screenHeight ;

    /**
     * X coordinate.
     **/
    int cornerTopLeftX ;

    /**
     * Y coordinate.
     **/
    int cornerTopLeftY ;

    /**
     * X coordinate.
     **/
    int cornerTopRightX ;

    /**
     * Y coordinate.
     **/
    int cornerTopRightY ;

    /**
     * X coordinate.
     **/
    int cornerBottomLeftX ;

    /**
     * Y coordinate.
     **/
    int cornerBottomLeftY ;

    /**
     * X coordinate.
     **/
    int cornerBottomRightX ;

    /**
     * Y coordinate.
     **/
    int cornerBottomRightY ;
} HdtCalibrationRec, *HdtCalibrationPtr;

/* ==== HdtFishNetLocator ==== */
/**
 * Symbols for HdtFishNetLocator.
 *
 * @see #HdtFishNetLocator__locateHorizontalPosition()
 * @see #HdtFishNetLocator__locateVerticalPosition()
 **/
#define HDT__FISH_NET_LOCATOR__STEP_COUNT_FOR_LOCATE (5)

/**
 * Supporting structure to approximate accurately the position of the cursor.
 **/
typedef struct _HdtFishNetLocator
{
	/**
	 * Width of the net
	 **/
	int width ;

	/**
	 * Height of the net
	 **/
	int height ;

	/**
	 * Values to convert raw coordinates into a first approximation of the
	 * cursor coordinate
	 * x_candidate = ((raw_x - rawToGridFactors[0][0])/rawToGridFactors[0][1])
	 * y_candidate = ((raw_y - rawToGridFactors[1][0])/rawToGridFactors[1][1])
	 **/
	fp8 rawToGridFactors[2][2] ;

	/**
	 * Coefficient defining the net along the horizontal axis
	 * Each top-level element contains three element coding a line :
	 * 0 = a*x + b*y + c
	 **/
	fp8* horizontalNet ;

	/**
	 * Coefficient defining the net along the vertical axis
	 * Each top-level element contains three element coding a line :
	 * 0 = a*x + b*y + c
	 **/
	fp8* verticalNet ;
} HdtFishNetLocatorRec, *HdtFishNetLocatorPtr ;


/* ==== API==== */
static void
HdtFishNetLocator__destroy(HdtFishNetLocatorPtr fishnet);

static int
HdtFishNetLocator__instanciate(HdtFishNetLocatorPtr fishnet, int width, int height);

static void
HdtFishNetLocator__init(HdtFishNetLocatorPtr fishnet, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4);


static int
HdtFishNetLocator__locateHorizontalPosition(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int step);

static int
HdtFishNetLocator__locateVerticalPosition(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int step);

/* ==== HdtConfiguration ==== */
/*
    __CONFIGURATION_NAME__ = Name of a parameter in xorg.conf
    __CONFIGURATION_DEFAULT__ = Default value for the corresponding parameter
 */

/**
 * Default number of packet to read in a row by default.
 *
 * @see #HdtConfiguration.packetCount
 **/
#define HDT__CONFIGURATION_NAME__PACKET_COUNT ("PacketCount")
#define HDT__CONFIGURATION_DEFAULT__PACKET_COUNT (6)

/**
 * Path to device.
 **/
#define HDT__CONFIGURATION_NAME__DEVICE ("Device")
#define HDT__CONFIGURATION_DEFAULT__DEVICE ("/dev/usb/hiddev0")

/**
 * Opcodes.
 **/
#define HDT__CONFIGURATION_NAME__OPCODE_PRESSURE ("OpcodePressure")
#define HDT__CONFIGURATION_DEFAULT__OPCODE_PRESSURE (0)

#define HDT__CONFIGURATION_NAME__OPCODE_X ("OpcodeX")
#define HDT__CONFIGURATION_DEFAULT__OPCODE_X (0)

#define HDT__CONFIGURATION_NAME__OPCODE_Y ("OpcodeY")
#define HDT__CONFIGURATION_DEFAULT__OPCODE_Y (0)

/**
 * Calibration.
 **/
#define HDT__CONFIGURATION_NAME__CALIBRATION__MODEL ("CalibrationModel")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__MODEL (0)
#define HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__MIN_MAX (0)
#define HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__FOUR_CORNER (1)


#define HDT__CONFIGURATION_NAME__CALIBRATION__MIN_X ("MinX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__MIN_X (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__MIN_Y ("MinY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__MIN_Y (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__MAX_X ("MaxX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__MAX_X (2000)

#define HDT__CONFIGURATION_NAME__CALIBRATION__MAX_Y ("MaxY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__MAX_Y (2000)


#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_X ("CornerTopLeftX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_LEFT_X (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_Y ("CornerTopLeftY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_LEFT_Y (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_X ("CornerTopRightX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_RIGHT_X (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_Y ("CornerTopRightY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_RIGHT_Y (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_X ("CornerBottomLeftX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_LEFT_X (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_Y ("CornerBottomLeftY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_LEFT_Y (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_X ("CornerBottomRightX")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_RIGHT_X (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_Y ("CornerBottomRightY")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_RIGHT_Y (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_WIDTH ("CornerScreenWidth")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER__SCREEN_WIDTH (0)

#define HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_HEIGHT ("CornerScreenHeight")
#define HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER__SCREEN_HEIGHT (0)


/**
 * Configuration of the driver.
 **/
typedef struct _HdtConfiguration {
    /**
     * Number of packet to read in a row by default.
     *
     * A packet is a pair (opCode, value). The device sends
     * <code>packetCount</code> packet to describe its new state.
     *
     * @see #HdtRawDataRec.pressure
     **/
    int packetCount ;

    /**
     * Opcode to watch for.
     **/
    HdtOpCodeRec opCodeSetup ;

    /**
     * Calibration settings.
     **/
    HdtCalibrationRec calibration ;

    /**
     * path of the device.
     **/
    char *device;
} HdtConfigurationRec, *HdtConfigurationPtr;

/* ==== API==== */

/**
 * Retrieve supported configuration parameters.
 *
 * @param dev the device interface from which options are retrieved.
 * @param hdtRawData the structure to fill.
 **/
static void
HdtConfiguration__retrieve(HdtConfigurationPtr configuration, IDevPtr dev) ;

/* ==== HdtDevice ==== */

typedef struct _HdtDevice {
    HdtRawDataRec myRawData ; /* Data as received from the device */
    HdtRawDataRec myTranslatedData ; /* Data suitable to post events */
    HdtConfigurationRec myConfiguration ;
    HdtFishNetLocatorRec myLocator ;
} HdtDeviceRec, *HdtDevicePtr;

/* ==== API==== */



#endif /** ifndef HIDTOUCH__H **/
